 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Table Grid Element                      *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyGrid
	 *
	 *	This is the Grid Element, the holder of cells.
	 *
	 * CSS Classes:
	 *
	 *	- .thyGrid
	 *	- .thyGrid_label
	 *	- .thyGrid_contents
	 *	- .thyGrid_row
	 *	- .thyGrid_row<n>
	 *	- .thyGrid_cell         - look at <thyGridCell> as all that extra classes
	 *	                          are also inherited
	 *	- .thyGrid_col_header
	 *	- .thyGrid_col
	 *	- .thyGrid_col<n>		- this is for defining column size
	 *
	 *	- .<object name>
	 *	- .<object name>_label
	 *	- .<object name>_contents
	 *	- .<object name>_row
	 *	- .<object name>_row<n>
	 *	- .<object name>_cell
	 *	- .<object name>_col_header
	 *	- .<object name>_col
	 *	- .<object name>_col<n>		- this is for defining column size
	 *
	 * Events:
	 *
	 *	selectrow - happens in the moment a row is selected
	 *	unselectrow - happens in the moment a row is unselected
	 *
	 * See Also:
	 *
	 *	<thyGridCell>
	 *
	 *	<thyCellCollection>
	 *
	 *	<thyPanel>
	 *
	 * Parameters:
	 *
	 *	name - The name of the widget
	 *	label - The label of the widget
	 */
	function thyGrid(name, label)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name,label);

		// Atributes
		this.selCellSelRow = false;
		this.initialized = false;
		this.gridContents = new thyGridContents(name+'_contents_table');


		this.rows = new thyVisualCollection(this.name+'_rowsCollection', this.gridContents);
		this.rowClasses = new thyCollection();

		// Initialization
		this.rows.setDefaultName('Row');
		this.addCSSClass('thyGrid');

		this.contents.addChild(this.gridContents);
	}

	p = dynapi.setPrototype('thyGrid', 'thyLabelPanel');

	/**
	 * Method: cleanUp
	 *
	 *	Removes every cell of the grid, including columns
	 *
	 */
	p.cleanUp = function ()
	{
		this.removeAllRows();
		this.removeAllColumns();
	}

	/**
	 * Method: reset
	 *
	 *	Removes all grid rows
	 *
	 */
	p.reset = function ()
	{
		this.removeAllRows();
	}
	
	/*************************************************************************\
	 *                         Group: Column Methods                         *
	\*************************************************************************/

	/**
	 * Method: insertColumn
	 *
	 *	Inserts a new column into the grid. This method (or <appendColumn>) can
	 *	*must* be called before any other to initialize the grid.
	 *
	 * Parameters:
	 *
	 *	title - The column title text
	 *	col   - The <thyCollection> of column contents. If this is the first inserted column,
	 *	        the collection can have any number of elements. If this is not the
	 *	        first, the number will be shrinked/grown to reflect the actual number
	 *	        of rows. This argument is *optional*.
	 *	index - The position this column must occupy. This argument is *optional*
	 *
	 */
	p.insertColumn = function (title, col, index)
	{
		var colIndex = this.appendColumn(title, col);
		if (index != null) this.moveColumn(colIndex, index);
	}

	/**
	 * Method: appendColumn
	 *
	 *	Inserts a new column in the next available position.
	 *
	 * Parameters:
	 *
	 *	title - The column title text
	 *	col   - The <thyCollection> of column contents. If this is the first inserted column,
	 *	        the collection can have any number of elements. If this is not the
	 *	        first, the number will be shrinked/grown to reflect the actual number
	 *	        of rows. This argument is *optional*.
	 *
	 */
	p.appendColumn = function (title, col)
	{
		if (title == null || typeof(title) == 'object')
		{
			throw({result: 'Invalid title', location: 'thyGrid.appendColumn'});
		}
		
		if (col != null && (typeof(col) != 'object' || !col.isFromClass('thyCollection')))
		{
			throw({result: 'Invalid collection', location: 'thyGrid.appendColumn'});
		}

		// Check for existing structure
		if (!this.rows.getElementsCount())
		{
			// Create header Row and inserts index cell
			this.rows.add(new thyGridRow('Row-1', -1), 'Row-1');
			this._addRowClasses(this.rows.getElementByIndex(0), -1);
			this._addRowCellsClasses(this.rows.getElementByIndex(0));
		}

		var headerRow = this.rows.getElementByIndex(0);
		var colIndex = headerRow.insertCell(title, null, 'h');

		if (col)
		{
			this.appendRow(col)
		}

		return colIndex-1;
	}

	/**
	 * Method: moveColumn
	 *
	 *	Moves a whole column from one position to another
	 *
	 * Parameters:
	 *
	 *	fromIndex - Original Column position
	 *	toIndex   - Destination Column position
	 *
	 * *THIS METHOD IS NOT IMPLEMENTED YET!!!*
	 *
	 */
	p.moveColumn = function (fromIndex, toIndex)
	{
	}

	/**
	 * Method: getColumn
	 *
	 *	Returns a <thyCollection> of all specified column contents
	 *
	 * Parameter:
	 *
	 *	index - The column position
	 *
	 */
	p.getColumn = function (index)
	{
		var i, nRows = this.rows.getElementsCount();
		var cells = new thyCollection();

		for (i=1; i<nRows; i++)
		{
			cells.add(this.rows.getElementByIndex(i).getCell(index));
		}

		return cells;
	}
	
	/**
	 * Method: getSelectedCols
	 *
	 *	Returns a <thyCollection> composed of other <thyCollections>, each one with the 
	 *	contents of a selected column.
	 *
	 * *THIS METHOD IS NOT IMPLEMENTED YET!!!*
	 *
	 */
	p.getSelectedColumns = function ()
	{
	}

	/**
	 * Method: getColumnsCount
	 *
	 *	Returns the number of columns this grid has
	 *
	 */
	p.getColumnsCount = function ()
	{
		var row = this.rows.getElementByIndex(0);
		if (row) return row.getCellsCount();
		return 0;
	}
	
	/**
	 * Method: removeColumn
	 *
	 *	Removes a column from Grid.
	 *
	 * Parameter:
	 *
	 *	index - The column position
	 *
	 */
	p.removeColumn = function (index)
	{
		if (!this.rows.getElementByIndex(0).getCellElement(index)) return;
		
		var i,row,nRows = this.rows.getElementsCount();

		for (i=0; i<nRows; i++)
		{
			row = this.rows.getElementByIndex(i);
			row.removeCell(index);
		}
	}

	/**
	 * Method: removeAllColumns
	 *
	 *	Removes all columns of the grid. This is actually the same
	 *	as turning the grid into its initial state.
	 *
	 */
	p.removeAllColumns = function ()
	{
		this.rows.cleanUp();
	}

	/**
	 * Method: searchColumn
	 *
	 *	Searches for a specified string in the specified column
	 *
	 * Parameters:
	 *
	 *	str - The string to be searched for
	 *	col - The column index
	 *
	 * Returns:
	 *
	 *	The row index
	 *
	 */
	p.searchColumn = function (str, col)
	{
		var i,content,cell,nRows = this.rows.getElementsCount();

		for (i=1; i<nRows; i++)
		{
			cell = this.rows.getElementByIndex(i).getCell(col);
			if (cell.indexOf(str) != null) return i-1;
		}
	}

	/*************************************************************************\
	 *                         Group: Row Methods                            *
	\*************************************************************************/

	/**
	 * Method: insertRow
	 *
	 *	Inserts a Row on the specified position (or at the end, if no index specified
	 *
	 * Parameters:
	 *
	 *	row - A <thyCollection> of row contents, indexed by column index
	 *	index - The position where this row should be inserted
	 *
	 */
	p.insertRow = function (row, index)
	{
		var rowIndex = this.appendRow(row);
		if (index != null) this.moveRow(rowIndex, index);
	}

	/**
	 * Method: appendRow
	 *
	 *	Inserts a row at the end of the grid.
	 *
	 * Paramenter:
	 *
	 *	row - A <thyCollection> of the Row contents, indexed by column index
	 *
	 */
	p.appendRow = function (row)
	{
		// Check if valid row
		if (!row || typeof(row) != 'object' || !(typeof(row.isFromClass) == 'function' && row.isFromClass('thyCollection')))
		{
			throw({result: 'Invalid Collection', location: 'thyGrid.appendRow'});
		}

		var i,nRows = this.rows.getElementsCount();
		if (!nRows)
		{
			throw({result: 'There must be Columns', location: 'thyGrid.appendRow'});
		}

		var nCols = row.getElementsCount();
		var nHeaderCols = this.rows.getElementByIndex(0).getCellsCount();
		var nColsToAdd = nHeaderCols - nCols > 0 ? nHeaderCols : nCols;
		
		var rowIdx = this.rows.add(new thyGridRow('Row'+(nRows-1), nRows-1), 'Row'+(nRows-1));
		var gRow = this.rows.getElementByIndex(nRows);

		if (this.selCellSelRow) gRow.setSelectCellSelectRow();

		this._addRowClasses(gRow, nRows-1);
		this._addRowCellsClasses(gRow);

		for (i=0; i<nHeaderCols; i++)
		{
			gRow.insertCell(row.getElementByIndex(i));
		}

		return nRows-1;
	}

	/**
	 * Method: moveRow
	 *
	 *	Move a Row from one position to another and refreshes CSS Classes
	 *
	 * Parameters:
	 *
	 *	fromIndex - Original Row position
	 *	toIndex   - Destination Row position
	 *
	 */
	p.moveRow = function (fromIndex, toIndex)
	{
		fromIndex++;
		toIndex++;
		var row;
		if (toIndex == null || fromIndex == null || 
		    typeof(toIndex) != 'number' || typeof(fromIndex) != 'number' ||
			toIndex >= this.rows.getElementsCount() ||
			!(row = this.rows.getElementByIndex(fromIndex))) throw({result: 'Invalid Index', location: 'thyGrid.moveRow'});
		
		this.rows.changeElementIndex({elm: row, index: toIndex, oldIndex: fromIndex});

		var i,nRows = this.rows.getElementsCount();

		for (i=fromIndex>toIndex ? toIndex:fromIndex; i<nRows; i++)
		{
			this._updateRowName(this.rows.getElementByIndex(i), i-1);
		}
	}

	/**
	 * Method: getRow
	 *
	 *	Returns a <thyCollection> of all specified row contents
	 *
	 * Parameter:
	 *
	 *	index - The row position
	 *
	 */
	p.getRow = function (index)
	{
		var row = this.rows.getElementByIndex(index+1);
		if (!row) return new thyCollection();
		
		var i,nCells = row.getCellsCount();
		var cells = new thyCollection();
		
		for (i=0; i<nCells; i++)
		{
			cells.add(row.getCell(i));
		}

		return cells;
	}

	/**
	 * Method: getRowElement
	 *
	 *	Returns the <thyGridRow> element that is in the specified position
	 *
	 *	When what you want is just the contents, please use <thyGrid>::getRow.
	 *
	 *	This method should be used *ONLY* by other widgets that wants to have
	 *	access to data only available through the <thyGridRow> object
	 *
	 * Parameter:
	 *
	 *	index - The row position
	 *
	 */
	p.getRowElement = function (index)
	{
		return this.rows.getElementByIndex(index+1);
	}

	/**
	 * Method: getSelectedRowsIndexes
	 *
	 *	Returns a <thyCollection> with the selected row indexes
	 *
	 */
	p.getSelectedRowsIndexes = function ()
	{
		var selRowsIndexes = new thyCollection();
		var i,nRows = this.rows.getElementsCount();

		for (i=1; i<nRows; i++)
		{
			if (this.rows.getElementByIndex(i).isSelected()) selRowsIndexes.add(i-1);
		}
		
		return selRowsIndexes;
	}

	/**
	 * Method: getSelectedRows
	 *
	 *	Returns a <thyCollection> composed of other <thyCollections>, each one with the 
	 *	contents of a selected row.
	 *
	 */
	p.getSelectedRows = function ()
	{
		var rows = new thyCollection();
		var i,j,nCells,cells,row,nRows = this.rows.getElementsCount();

		for (i=1; i<nRows; i++)
		{
			row = this.rows.getElementByIndex(i);
			if (row.isSelected()) rows.add(this.getRow(i-1));
		}
		
		return rows;
	}

	/**
	 * Method: getRowsCount
	 *
	 *	Returns the number of rows in Grid
	 *
	 */
	p.getRowsCount = function ()
	{
		return this.rows.getElementsCount()-1;
	}

	/**
	 * Method: getAllRows
	 *
	 *	Return a <thyCollection> with other <thyCollections> containing
	 *	the elements of the entire <thyGrid>
	 *
	 */
	p.getAllRows = function ()
	{
		var i,nRows = this.getRowsCount();
		var allRows = new thyCollection();
		
		for (i=0; i<nRows; i++)
		{
			allRows.add(this.getRow(i));
		}

		return allRows;
	}

	/**
	 * Method: removeAllRows
	 *
	 *	Removes all rows from Grid
	 *
	 */
	p.removeAllRows = function ()
	{
		var i,nRows = this.rows.getElementsCount();
		for (i=nRows-1; i>0; i--)
		{
			this.rows.removeByIndex(i);
		}
	}
	
	/**
	 * Method: removeSelectedRows
	 *
	 *	Remove all selected Rows from Grid
	 *
	 */
	p.removeSelectedRows = function ()
	{
		var i,lastRow,nRows = this.rows.getElementsCount();
		for (i=nRows-1; i>0; i--)
		{
			if (this.rows.getElementByIndex(i).isSelected())
			{
				this.rows.removeByIndex(i);
				lastRow = i;
			}
		}

		nRows = this.rows.getElementsCount();
		for (i=lastRow; i<nRows; i++)
		{
			this._updateRowName(this.rows.getElementByIndex(i), i-1);
		}
	}
	
	/**
	 * Method: removeRow
	 *
	 *	Remove specified row from Grid and re-index rows
	 *
	 * Parameter:
	 *
	 *	index - The row index
	 *
	 */
	p.removeRow = function (index)
	{
		var row,idx = index+1;
		this.rows.removeByIndex(idx);

		var i,nRows = this.rows.getElementsCount();
		for (i=idx; i<nRows; i++)
		{
			this._updateRowName(this.rows.getElementByIndex(i),i-1);
		}
	}

	/**
	 * Method: setSelectCellSelectRow
	 *
	 *	Always selects a row instead of just a cell.
	 *	
	 */
	p.setSelectCellSelectRow = function ()
	{
		this.selCellSelRow = true;
		var i,nRows = this.rows.getElementsCount();

		for (i=0; i<nRows; i++)
		{
			this.rows.getElementByIndex(i).setSelectCellSelectRow();
		}
	}

	/**
	 * Method:unsetSelectCellSelectRow
	 *
	 *	Always selects the cell, instead of the whole row
	 *
	 */
	p.unsetSelectCellSelectRow = function ()
	{
		this.selCellSelRow = false;
		var i,nRows = this.rows.getElementsCount();

		for (i=0; i<nRows; i++)
		{
			this.rows.getElementByIndex(i).unsetSelectCellSelectRow();
		}
	}
	
	/*************************************************************************\
	 *                         Group: Cell Methods                           *
	\*************************************************************************/

	/**
	 * Method: getCell
	 *
	 *	Returns the contents of a specified Cell
	 *
	 * Parameters:
	 *
	 *	rIndex - The row index
	 *	cIndex - The col index
	 *
	 */
	p.getCell = function (rIndex, cIndex)
	{
		var row = this.rows.getElementByIndex(rIndex+1);
		if (row) return row.getCell(cIndex);
		return null;
	}

	/**
	 * Method: getSelectedCells
	 *
	 *	Returns a <thyCollection> with the contents of the selected cells
	 *
	 */
	p.getSelectedCells = function ()
	{
		var cells = new thyCollection();
		var i,row,nRows = this.rows.getElementsCount();
		var j,selCells,nSelCells;

		for (i=1; i<nRows; i++)
		{
			row = this.rows.getElementByIndex(i);
			selCells = row.getSelectedCells();
			nSelCells = selCells.getElementsCount();
			for (j=0;j<nSelCells; j++)
			{
				cells.add(selCells.getElementByIndex(j));
			}
		}
		
		return cells;
	}

	/**
	 * Method: getCellElement
	 *
	 *	Returns the <thyGridCell> element that resides on the specified location
	 *
	 *	If what you want is just the contents of the <thyGridCell> element, please
	 *	use <thyGrid>::getCell.
	 *
	 *	This method should be used *ONLY* when the user wants to manipulate any
	 *	property of the cell element.
	 *
	 * Parameters:
	 *
	 *	rIndex - The row index
	 *	cIndex - The col index
	 *
	 */
	p.getCellElement = function (rIndex, cIndex)
	{
		var row;
		if (row = this.rows.getElementByIndex(rIndex+1)) return row.getCellElement(cIndex);
	}

	/**
	 * Method: changeCell
	 *
	 *	Change the contents of a specified cell.
	 *
	 * Parameters:
	 *
	 *	content - The new Cell Content
	 *	rIndex  - The row index
	 *	cIndex  - The col index
	 *
	 */
	p.changeCell = function (content, rIndex, cIndex)
	{
		var row, cell;

		if (!(row = this.getRowElement(rIndex))) return;
		
		if (!(cell = row.getCellElement(cIndex))) return;

		cell.populate(content);
	}

	/*************************************************************************\
	 *                       Group: Private Methods                          *
	\*************************************************************************/

	/**
	 * Method: _getDOMChildrenHolder
	 *
	 *	Returns the DOM object that holds children DOM objects
	 *
	 */
	p._getDOMChildrenHolder = function ()
	{
		if (!this._created) return null;

		return this.elm.childNodes[0];
	}
	
	p._thyLabelPanel_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Overwritten method that adds the specified class to main, rows and cells
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be added
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyLabelPanel_addCSSClass(cssClass);
		if (!this.rows) return;

		this.rowClasses.add(cssClass);
		var i,row,nRows = this.rows.getElementsCount();
		for (i=0; i<nRows; i++)
		{
			row = this.rows.getElementByIndex(i);
			this._addRowClasses(row, i-1);
			this._addRowCellsClasses(row);
		}
	}

	p._thyLabelPanel_removeCSSClass = p._removeCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Overwritten method that removes the specified class from main, rows and cells
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyLabelPanel_removeCSSClass(cssClass);
		if (!this.rows) return;

		this.rowClasses.remove(cssClass);
		var i,row,nRows = this.rows.getElementsCount();
		for (i=0; i<nRows; i++)
		{
			row = this.rows.getElementByIndex(i);
			this._removeRowClasses(row, i-1);
		}
	}
	
	/**
	 * Method: _addRowClasses
	 *
	 *	Add all classes that were appended to <thyGrid> to a Row
	 *
	 * Parameters:
	 *
	 *	row - The <thyGridRow> element that will have the classes
	 *	rowIndex - The index of the row
	 *
	 */
	p._addRowClasses = function (row, rowIndex)
	{
		var i, cssClass, nClasses = this.rowClasses.getElementsCount();

		for (i=0; i<nClasses; i++)
		{
			cssClass = this.rowClasses.getElementByIndex(i);
			row._addCSSClass(cssClass+'_row');
			row._addCSSClass(cssClass+'_row'+rowIndex);
		}
		row.setCSSClass(row.cssActualSelection);
	}

	/**
	 * Method: _addRowCellsClasses
	 *
	 *	Adds all classes that were appended to <thyGrid> to the
	 *	cells of a row with their correct values
	 *
	 * Parameters:
	 *
	 *	row - The <thyGridRow> element whose cells will have their classes
	 *	      set.
	 *
	 */
	p._addRowCellsClasses = function (row)
	{
		var i, cssClass, nClasses = this.rowClasses.getElementsCount();

		for (i=0; i<nClasses; i++)
		{
			cssClass = this.rowClasses.getElementByIndex(i);
			row._addCellCSSClass(cssClass+'_cell');
			row._addCellCSSClass(cssClass+'_col');
			row._addCellCSSClass(this.name+'_col');
			row._addCellIndexedCSSClass(cssClass+'_col');
			row._addCellIndexedCSSClass(this.name+'_col');
			row._updateAllClasses();
		}
	}
	
	/**
	 * Method: _removeRowClasses
	 *
	 *	Removes all classes that were appended to <thyGrid> from a Row
	 *
	 * Parameters:
	 *
	 *	row - The <thyGridRow> element that will have its classes removed
	 *	rowIndex - The index of the Row
	 *
	 */
	p._removeRowClasses = function (row, rowIndex)
	{
		var i, cssClass, nClasses = this.rowClasses.getElementsCount();

		for (i=0; i<nClasses; i++)
		{
			cssClass = this.rowClasses.getElementByIndex(i);
			row._removeCSSClass(cssClass+'_row');
			row._removeCSSClass(cssClass+'_row'+rowIndex);
		}
		row.setCSSClass(row.cssActualSelection);
	}
	
	/**
	 * Method: _removeRowCellsClasses
	 *
	 *	Removes all classes that were appended to <thyGrid> from all Cells
	 *	of a Row
	 *
	 * Parameter:
	 *
	 *	row - The <thyGridRow> element that will have its classes removed
	 *
	 */
	p._removeRowCellsClasses = function (row)
	{
		var i, cssClass, nClasses = this.rowClasses.getElementsCount();

		for (i=0; i<nClasses; i++)
		{
			cssClass = this.rowClasses.getElementByIndex(i);
			row._removeAllClasses();
			row._removeCellCSSClass(cssClass+'_cell');
			row._removeCellCSSClass(cssClass+'_col');
			row._removeCellCSSClass(this.name+'_col');
			row._removeCellIndexedCSSClass(cssClass+'_col');
			row._removeCellIndexedCSSClass(this.name+'_col');
			row._updateAllClasses();
		}
	}
	
	/**
	 * Method: _updateRowName
	 *
	 *	Updates Row name to reflect its actual position
	 *
	 * Parameter:
	 *
	 *	row - The <thyGridRow> that must have its name updated
	 *	rowIndex - The position of the Row
	 *
	 */
	p._updateRowName = function (row, rowIndex)
	{
		this._removeRowClasses(row, row.getIndex());
		row.setName('Row'+rowIndex);
		row.setIndex(rowIndex);
		this._addRowClasses(row, rowIndex);
		this.rows.changeElementName({elm: row, name: 'Row'+rowIndex});
	}

	/**
	 * Method: _populateContents
	 *
	 *	Clear up the <thyGrid> and inserts new specified contents
	 *
	 * Parameters:
	 *
	 *	data - Can be of 2 types:
	 *
	 *	- An array of arrays, each one with the contents of all cells 
	 *	  of that row.
	 *	- An object containing the columns or the rows or both
	 *
	 */
	p._populateContents = function (data)
	{
		var i, j, cells, cell, rowData;
		var selected = data.selectedRows;

		if (typeof(data) != 'object') return;
		
		if (!selected || data.columns || data.rows)
		{
			this.removeAllRows();

			if (data.columns || data.rows)
			{
				var cols = data.columns;
				var rows = data.rows;
				
				if (cols)
				{
					this.removeAllColumns();
					
					for (i=0; i<cols.length; i++)
					{
						this.appendColumn(cols[i]);
					}
				}

				if (rows)
				{
					data = rows;
				}
				else
				{
					data.length = 0;
				}
			}
			
			for (i=0; i<data.length; i++)
			{
				rowData = data[i];
				cells = new thyCollection('Row'+i);
				
				for (j = 0; j<rowData.length; j++)
				{
					cells.add(rowData[j]);
				}
				
				this.appendRow(cells);
			}
		}

		if (selected)
		{
			var i,nSel = selected.length;
			var j,nRows = this.getRowsCount();

			for (i=0; i<nSel; i++)
			{
				this.getRowElement(selected[i]).select();
			}
		}
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Return all the contents of a grid
	 *
	 */
	p._sweepOutContents = function ()
	{
		var i,nRows = this.getRowsCount();
		data = {rows: {}, selectedRows: {}};

		for (i=0; i<nRows; i++)
		{
			data.rows[i] = this.getRow(i).getIndexedElements();
		}

		data.selectedRows = this.getSelectedRowsIndexes().getIndexedElements();

		return data;
	}
	
	/****************************************************************************\
	 *                                Events                                    *
	\****************************************************************************/
